'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = wrapRoute;

var _reactRedux = require('react-redux');

var _reduxFeatures = require('redux-features');

var _reselect = require('reselect');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var babelPluginFlowReactPropTypes_proptype_RouteComponent = require('react-router').babelPluginFlowReactPropTypes_proptype_RouteComponent || require('prop-types').any;

var babelPluginFlowReactPropTypes_proptype_RouterState = require('react-router').babelPluginFlowReactPropTypes_proptype_RouterState || require('prop-types').any;

var babelPluginFlowReactPropTypes_proptype_PlainRoute = require('react-router').babelPluginFlowReactPropTypes_proptype_PlainRoute || require('prop-types').any;

var babelPluginFlowReactPropTypes_proptype_Store = require('redux').babelPluginFlowReactPropTypes_proptype_Store || require('prop-types').any;

var babelPluginFlowReactPropTypes_proptype_FeatureState = require('redux-features').babelPluginFlowReactPropTypes_proptype_FeatureState || require('prop-types').any;

var babelPluginFlowReactPropTypes_proptype_Feature = require('redux-features').babelPluginFlowReactPropTypes_proptype_Feature || require('prop-types').any;

var babelPluginFlowReactPropTypes_proptype_Options = require('./wrapRoute').babelPluginFlowReactPropTypes_proptype_Options || require('prop-types').any;

function wrapRoute(options) {
  var store = options.store,
      featureId = options.featureId,
      featureName = options.featureName,
      getFeatureStates = options.getFeatureStates,
      getFeatures = options.getFeatures,
      isServer = options.isServer,
      getRoute = options.getRoute,
      getRoutes = options.getRoutes;

  var rematchRoutes = isServer ? null : options.rematchRoutes;
  var route = typeof options.route === 'function' ? options.route(store) : options.route;
  var result = (0, _extends3.default)({}, route);

  var selectFeatureState = (0, _reselect.createSelector)(getFeatureStates, function (featureStates) {
    return featureStates[featureId];
  });
  var selectFeature = (0, _reselect.createSelector)(getFeatures, function (features) {
    return features[featureId];
  });

  var FeatureStateAlert = options.FeatureStateAlert && (0, _reactRedux.connect)((0, _reselect.createSelector)(selectFeatureState, function (featureState) {
    return { featureId: featureId, featureName: featureName, featureState: featureState };
  }))(options.FeatureStateAlert);

  var selectLoadedRoute = (0, _reselect.createSelector)(selectFeature, getRoute ? (0, _reselect.createSelector)(getRoute, function (route) {
    return typeof route === 'function' ? route(store) : route || {};
  }) : (0, _reselect.createSelector)(getRoutes || function () {
    return [];
  }, function (routes) {
    if (!routes) return {};
    if (!Array.isArray(routes)) routes = [routes];
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = (0, _getIterator3.default)(routes), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var other = _step.value;

        if (typeof other === 'function') other = other(store);
        if (other.path === route.path) return other;
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return {};
  }));

  delete result.component;
  result.getComponent = function (nextState, callback) {
    var featureState = selectFeatureState(store.getState());

    if (featureState !== 'LOADED') {
      callback(null, FeatureStateAlert);
      return;
    }
    var loadedRoute = selectLoadedRoute(store.getState());
    if (loadedRoute.getComponent) loadedRoute.getComponent(nextState, callback);else if (loadedRoute.component) callback(null, loadedRoute.component);else callback(null, null);
  };

  function wrapProperty(valueName, getterName, defaultValue) {
    delete result[valueName];
    result[getterName] = function (nextState, callback) {
      var promise = selectFeatureState(store.getState()) === 'NOT_LOADED' ? store.dispatch((0, _reduxFeatures.loadFeature)(featureId)) : null;
      function done() {
        var loadedRoute = selectLoadedRoute(store.getState());
        if (loadedRoute[getterName]) return loadedRoute[getterName](nextState, callback);else if (loadedRoute[valueName]) callback(null, loadedRoute[valueName]);else callback(null, isServer ? null : defaultValue);
      }
      if (promise) {
        if (isServer) {
          promise.then(done).catch(callback);
          return;
        } else if (rematchRoutes) promise.then(function () {
          return rematchRoutes(store);
        });
      }
      return done();
    };
  }

  wrapProperty('components', 'getComponents', null);
  wrapProperty('indexRoute', 'getIndexRoute', null);
  wrapProperty('childRoutes', 'getChildRoutes', [{ path: '*' }]);

  result.onEnter = function (nextState, replace, callback) {
    var promise = selectFeatureState(store.getState()) === 'NOT_LOADED' ? store.dispatch((0, _reduxFeatures.loadFeature)(featureId)) : null;
    function done() {
      var loadedRoute = selectLoadedRoute(store.getState());
      if (loadedRoute.onEnter) return loadedRoute.onEnter(nextState, replace, callback);else callback(null);
    }
    if (promise) {
      if (isServer) {
        promise.then(done).catch(callback);
        return;
      } else if (rematchRoutes) promise.then(function () {
        return rematchRoutes(store);
      });
    }
    return done();
  };
  result.onChange = function (prevState, nextState, replace, callback) {
    var loadedRoute = selectLoadedRoute(store.getState());
    if (loadedRoute.onChange) return loadedRoute.onChange(prevState, nextState, replace, callback);else callback();
  };
  result.onLeave = function (prevState) {
    var loadedRoute = selectLoadedRoute(store.getState());
    if (loadedRoute.onLeave) return loadedRoute.onLeave(prevState);
  };

  return result;
}